from pylab import *

import pandas as pd
import statsmodels.api as sm

__all__ = [ 'features', 'labels', 'featuref', 'dataf', 
            'scatter_binary', 'plot_model2d', 
            'plot_fit2d' ]

features = array([
  # Positive (15)
  [ 23735,  41 ],
  [ 41440,  58 ],
  [ 59339,  40 ],
  [ 62840,  27 ],
  [ 71206,  12 ],
  [ 56031,  60 ],
  [ 59339,  75 ],
  [ 67510,  64 ],
  [ 70623,  84 ],
  [ 77237,  57 ],
  [ 78794,  74 ],
  [ 88521,  63 ],
  [ 91829,  78 ],
  [ 94942,  69 ],
  [ 94942,  55 ],
  # Negative (18)
  [ 11479,  29 ],
  [ 16342,  14 ],
  [ 17899,  57 ],
  [ 21206,  25 ],
  [ 24319,  11 ],
  [ 29377,  57 ],
  [ 34047,  29 ],
  [ 37354,  65 ],
  [ 40661,  52 ],
  [ 38911,  41 ],
  [ 34047,  29 ],
  [ 40661,  18 ],
  [ 53502,  43 ],
  [ 55253,  18 ],
  [ 65175,  12 ],
  [ 71401,  30 ],
  [ 72957,  72 ],
  [ 84436,  56 ],
], float)

labels = array([1] * 15 + [0] * 18)

featuref = pd.DataFrame(features, columns=('Income', 'Age'))

dataf = pd.DataFrame(c_[features, labels], columns=('Income', 'Age', 'Label'))
dataf['Label'] = dataf['Label'].astype(int)  # Change type of Label column

def scatter_binary(data, labels, pos = 1, neg = 0, ax = None):
  if ax is None:
    ax = gca()
  pos_mask = (labels == pos)
  neg_mask = (labels == neg)
  ax.plot(data[pos_mask,0], data[pos_mask,1], 'og',
          data[neg_mask,0], data[neg_mask,1], 'xr')
  draw_if_interactive()


def plot_model2d(model, 
                 data = None, xlim = None, ylim = None, npts = 50,
                 value_func = 'predict',  # Name of model method
                 surface = False, zlim = None,
                 add_constant = False, threshold = None,
                 cmap = cm.Paired, cbar = False, ax = None):
  # Check arguments
  if data is not None:
    if xlim is not None or ylim is not None:
      raise ValueError, "Cannot specify limits when data is given"
    xlim = (data[:,0].min(), data[:,0].max())
    ylim = (data[:,1].min(), data[:,1].max())
  if xlim is None or ylim is None:
    raise ValueError, "Need to either specify limits or provide data"
  if ax is None:
    ax = gca() if not surface else gca(projection='3d')

  # Estimate on grid
  xx, yy = meshgrid(arange(xlim[0], xlim[1], float(xlim[1]-xlim[0])/(npts-1)), 
                    arange(ylim[0], ylim[1], float(ylim[1]-ylim[0])/(npts-1)))
  xy = c_[xx.ravel(), yy.ravel()]
  if add_constant:
    xy = sm.add_constant(xy)
  func = getattr(model, value_func)
  z = func(xy)
  if threshold is not None:
    z = (z >= threshold).astype(float)
  z = z.reshape(xx.shape)

  # Plot surface / contours
  if surface:
    surf = ax.plot_surface(xx, yy, z, rstride=2, cstride=2, cmap=cmap, alpha=0.75)
    #contour = ax.contourf(xx, yy, z, cmap = cmap, zdir='z')
    if zlim is not None:
      ax.set_zlim(zlim[0], zlim[1])
  else:
    contour = ax.contourf(xx, yy, z, cmap = cmap)
  if cbar:
    colorbar(contour, ax=ax)
  draw_if_interactive()


def plot_fit2d(model, data, labels, npts = 200, cbar = False,
               value_func = 'predict', surface = False,
               add_constant = False, threshold = None, fig = None):
  if fig is None: 
    fig = figure()
  ax = fig.gca() if not surface else fig.gca(projection='3d')
  scatter_binary(data, labels, ax=ax)
  plot_model2d(model, data, add_constant=add_constant, threshold = threshold, 
               value_func = value_func, surface = surface,
               npts=npts, cbar=cbar, ax=ax)
  fig.show()

